@@ -2,7 +2,7 @@
 # Usage: $0 [<options>] [<input> ...]
 #        $0 [<options>] --play <cmdlog> [--batch] [-w <waitsecs>] [-o <output>] [field=value ...]
 
-__version__ = '3.0.1'
+__version__ = '3.2'
 __version_info__ = 'saul.pw/VisiData v' + __version__
 
 from copy import copy
@@ -16,7 +16,7 @@ import signal
 import warnings
 import builtins  # to override print
 
-from visidata import vd, options, run, BaseSheet, AttrDict
+from visidata import vd, options, run, BaseSheet, AttrDict, stacktrace
 from visidata import Path
 from visidata.settings import _get_config_file
 import visidata
@@ -37,16 +37,20 @@ def eval_vd(logpath, *args, **kwargs):
     'Instantiate logpath with args/kwargs replaced and replay all commands.'
     log = logpath.read_text()
     if args or kwargs:
-        if logpath.ext in ['vdj', 'json', 'jsonl']:
+        if logpath.ext in ['vdj', 'json', 'jsonl'] or logpath is vd.stdinSource:
             from string import Template
             log = Template(log).safe_substitute(**kwargs)
         else:
             log = log.format(*args, **kwargs)
 
     src = Path(logpath.given, fptext=io.StringIO(log), filesize=len(log))
-    vs = vd.openSource(src, filetype=src.ext)
+    if logpath is vd.stdinSource:
+        # replay from stdin only supports .vdj
+        vs = vd.openSource(src, filetype='vdj')
+    else:
+        vs = vd.openSource(src, filetype=src.ext)
     vs.name += '_vd'
-    vs.reload()
+    vd.sync(vs.reload())
     vs.vd = vd
     return vs
 
@@ -116,13 +120,17 @@ def parsePos(vd, arg:str, inputs=None):
 def outputProgressEvery(vd, sheet, seconds:float=0.5):
     import time
     t0 = time.time()
-    while True:
-        time.sleep(seconds)
+
+    while not vd.currentReplay:
+        time.sleep(.1)
+
+    while vd.currentReplay:
         t = time.time()
         print(f'\r[{t-t0:.1f}s] ', end='', file=sys.stderr)
         if sheet:
             print(f'{sheet.progressPct}  ', end='', file=sys.stderr)
         sys.stderr.flush()
+        time.sleep(seconds)
 
 @visidata.VisiData.api
 def moveToPos(vd, sources, startsheets, startrow, startcol):
@@ -181,7 +189,7 @@ def main_vd():
         vd.warning(e)
 
     warnings.showwarning = vd.warning
-    vd.printout = builtins.print
+    vd.printerr = lambda *args: builtins.print(*args, file=sys.stderr)
 
     flPipedInput = not sys.stdin.isatty()
     flPipedOutput = not sys.stdout.isatty()
@@ -216,6 +224,8 @@ def main_vd():
         elif arg in ['--']:
             optsdone = True
         elif arg == '-':
+            if not flPipedInput:
+                vd.fail('to use stdin as a data source, data must be piped into it')
             inputs.append((vd.stdinSource, copy(current_args)))
         elif arg in ['-g', '--global']:
             flGlobal = True
@@ -248,10 +258,8 @@ def main_vd():
             current_args[optname] = optval
             if flGlobal:
                 global_args[optname] = optval
-
         elif arg.startswith('+'):  # position cursor at start
             after_config.append((vd.moveToPos, *vd.parsePos(arg[1:], inputs=inputs)))
-
         elif current_args.get('play', None) and '=' in arg:
             # parse 'key=value' pairs for formatting cmdlog template in replay mode
             k, v = arg.split('=', maxsplit=1)
@@ -278,9 +286,9 @@ def main_vd():
     vd.domotd()
 
     if args.batch:
-        options.undo = False
-        options.quitguard = False
-        vd.editline = lambda *args, **kwargs: ''
+        if not vd.options.interactive:
+            options.undo = False
+            options.quitguard = False
         vd.execAsync = vd.execSync  # disable async
 
     for cmd in (args.preplay or '').split():
@@ -328,7 +336,6 @@ def main_vd():
         if args.batch:
             if sources:
                 vd.push(sources[0])
-                sources[0].reload()
 
         for (f, *parms) in after_config:
             f(sources, *parms)
@@ -337,21 +344,22 @@ def main_vd():
             run(vd.sheets[0])
     else:
         if args.play == '-':
+            if vd.stdinSource.fptext.isatty():
+                vd.fail('replay commands must come by pipe, not by terminal')
             vdfile = vd.stdinSource
-            vdfile.name = 'stdin.vd'
         else:
             vdfile = Path(args.play)
 
         vs = eval_vd(vdfile, *fmtargs, **fmtkwargs)
-        vd.sync(vs.reload())
         if args.batch:
             if not args.debug:
                 vd.outputProgressThread = visidata.VisiData.execAsync(vd, vd.outputProgressEvery, vs, seconds=0.5, sheet=BaseSheet())  #1182
+            vd.reloadMacros()
             if vd.replay_sync(vs):  # error
                 return 1
 
             if vd.options.interactive:
-                vd.editline = lambda *args, vd=vd, **kwargs: visidata.VisiData.editline(vd, *args, **kwargs)
+                vd.options.batch = False  #2639
                 vd.execAsync = lambda *args, vd=vd, **kwargs: visidata.VisiData.execAsync(vd, *args, **kwargs)
                 run()
         else:
@@ -364,7 +372,7 @@ def main_vd():
 
     saver_threads = [t for t in vd.unfinishedThreads if t.name.startswith('save_')]
     if saver_threads:
-        vd.printout('finishing %d savers' % len(saver_threads))
+        vd.printerr('finishing %d savers' % len(saver_threads))
         vd.sync(*saver_threads)
 
     vd._stdout.flush()
@@ -381,9 +389,12 @@ def vd_cli():
         if vd.options.debug:
             raise
     except FileNotFoundError as e:
-        print(e)
+        print(e, file=sys.stderr)
         if options.debug:
             raise
+    except Exception as e:
+        for l in stacktrace(): #show the stack trace without carets
+            print(l, file=sys.stderr)
 
     sys.stderr.flush()
     sys.stdout.flush()
